# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# -----------------------------------

# pylint: disable=line-too-long
# This is to allow complete package description on PyPI
"""
Core components of the Microsoft Microsoft Agents M365 Copilot Python SDK consisting of HTTP Client and a configurable middleware pipeline (Preview).
"""
from ._constants import SDK_VERSION
from ._enums import APIVersion, FeatureUsageFlag, NationalClouds
from .authentication import AzureIdentityAuthenticationProvider
from .base_request_adapter import BaseMicrosoftAgentsM365CopilotRequestAdapter
from .client_factory import MicrosoftAgentsM365CopilotClientFactory
from .middleware import (
    AsyncMicrosoftAgentsM365CopilotTransport,
    MicrosoftAgentsM365CopilotRequestContext,
    MicrosoftAgentsM365CopilotTelemetryHandler,
    MicrosoftAgentsM365CopilotTelemetryHandlerOption,
)

__all__ = [
    "MicrosoftAgentsM365CopilotClientFactory", "BaseMicrosoftAgentsM365CopilotRequestAdapter",
    "AzureIdentityAuthenticationProvider", "FeatureUsageFlag", "NationalClouds", "APIVersion",
    "MicrosoftAgentsM365CopilotTelemetryHandlerOption", "MicrosoftAgentsM365CopilotRequestContext",
    "AsyncMicrosoftAgentsM365CopilotTransport", "MicrosoftAgentsM365CopilotTelemetryHandler"
]

__version__ = SDK_VERSION



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""
Application constants. All defaults can be changed when initializing a client
via the MicrosoftAgentsM365CopilotClient or HttpClientFactory
"""
DEFAULT_REQUEST_TIMEOUT = 100
DEFAULT_CONNECTION_TIMEOUT = 30
# The SDK version
# x-release-please-start-version
SDK_VERSION = '1.0.0-preview.1'
# x-release-please-end
MS_DEFAULT_SCOPE = 'https://graph.microsoft.com/.default'



from typing import Optional

import httpx
from kiota_abstractions.authentication import AuthenticationProvider
from kiota_abstractions.serialization import (
    ParseNodeFactory,
    ParseNodeFactoryRegistry,
    SerializationWriterFactory,
    SerializationWriterFactoryRegistry,
)
from kiota_http.httpx_request_adapter import HttpxRequestAdapter

from .client_factory import MicrosoftAgentsM365CopilotClientFactory


class BaseMicrosoftAgentsM365CopilotRequestAdapter(HttpxRequestAdapter):

    def __init__(
        self,
        authentication_provider: AuthenticationProvider,
        parse_node_factory: Optional[ParseNodeFactory] = None,
        serialization_writer_factory: Optional[SerializationWriterFactory] = None,
        http_client: Optional[httpx.AsyncClient] = None
    ) -> None:
        if parse_node_factory is None:
            parse_node_factory = ParseNodeFactoryRegistry()
        if serialization_writer_factory is None:
            serialization_writer_factory = SerializationWriterFactoryRegistry()
        if http_client is None:
            http_client = MicrosoftAgentsM365CopilotClientFactory.create_with_default_middleware()
        super().__init__(
            authentication_provider=authentication_provider,
            parse_node_factory=parse_node_factory,
            serialization_writer_factory=serialization_writer_factory,
            http_client=http_client
        )



import http
import json
import platform

import httpx
from kiota_http.middleware import BaseMiddleware
from urllib3.util import parse_url

from .._constants import SDK_VERSION
from .._enums import APIVersion, NationalClouds
from .async_transport import AsyncMicrosoftAgentsM365CopilotTransport
from .options import MicrosoftAgentsM365CopilotTelemetryHandlerOption
from .request_context import MicrosoftAgentsM365CopilotRequestContext


class MicrosoftAgentsM365CopilotRequest(httpx.Request):
    context: MicrosoftAgentsM365CopilotRequestContext


class MicrosoftAgentsM365CopilotTelemetryHandler(BaseMiddleware):
    """Middleware component that attaches metadata to a MicrosoftAgentsM365CopilotRequest request in
    order to help the SDK team improve the developer experience.
    """

    def __init__(self, options=MicrosoftAgentsM365CopilotTelemetryHandlerOption(), **kwargs):
        """Create an instance of MicrosoftAgentsM365CopilotTelemetryHandler

        Args:
            options (MicrosoftAgentsM365CopilotTelemetryHandlerOption, optional): The telemetry handler
            options value. Defaults to MicrosoftAgentsM365CopilotTelemetryHandlerOption
        """
        super().__init__()
        self.options = options

    async def send(
        self, request: MicrosoftAgentsM365CopilotRequest,
        transport: AsyncMicrosoftAgentsM365CopilotTransport
    ):
        """Adds telemetry headers and sends the http request.
        """
        current_options = self._get_current_options(request)

        if self.is_graph_url(request.url):
            self._add_client_request_id_header(request)
            self._append_sdk_version_header(request, current_options)
            self._add_host_os_header(request)
            self._add_runtime_environment_header(request)

        response = await super().send(request, transport)
        return response

    def _get_current_options(
        self, request: httpx.Request
    ) -> MicrosoftAgentsM365CopilotTelemetryHandlerOption:
        """Returns the options to use for the request.Overrides default options if
        request options are passed.

        Args:
            request (httpx.Request): The prepared request object

        Returns:
            MicrosoftAgentsM365CopilotTelemetryHandlerOption: The options to used.
        """
        current_options = self.options
        request_options = request.context.middleware_control.get(              # type:ignore
            MicrosoftAgentsM365CopilotTelemetryHandlerOption.get_key()
        )
        # Override default options with request options
        if request_options:
            current_options = request_options

        return current_options

    def is_graph_url(self, url):
        """Check if the request is made to a graph endpoint. We do not add telemetry headers to
        non-graph endpoints"""
        endpoints = set(item.value for item in NationalClouds)

        base_url = parse_url(str(url))
        endpoint = f"{base_url.scheme}://{base_url.netloc}"
        return endpoint in endpoints

    def _add_client_request_id_header(self, request) -> None:
        """Add a client-request-id header with GUID value to request"""
        request.headers.update({'client-request-id': f'{request.context.client_request_id}'})

    def _append_sdk_version_header(self, request, options) -> None:
        """Add SdkVersion request header to each request to identify the language and
        version of the client SDK library(s).
        Also adds the featureUsage value.
        """
        core_library_name = f'microsoft-agents-m365copilot-core/{SDK_VERSION}'
        service_lib_name = ''

        if options.api_version == APIVersion.v1:
            service_lib_name = f'microsoft-agents-m365copilot/{options.sdk_version}'
        if options.api_version == APIVersion.beta:
            service_lib_name = f'microsoft-agents-m365copilot-beta/{options.sdk_version}'

        if service_lib_name:
            telemetry_header_string = f'{service_lib_name}, '\
                f'{core_library_name} (featureUsage={request.context.feature_usage})'
        else:
            telemetry_header_string = f'{core_library_name} '\
                '(featureUsage={request.context.feature_usage})'

        if 'sdkVersion' in request.headers:
            sdk_version = request.headers.get('sdkVersion')
            if not sdk_version == telemetry_header_string:
                request.headers.update({'sdkVersion': telemetry_header_string})
        else:
            request.headers.update({'sdkVersion': telemetry_header_string})

    def _add_host_os_header(self, request) -> None:
        """
        Add HostOS request header to each request to help identify the OS
        on which our client SDK is running on
        """
        system = platform.system()
        version = platform.version()
        host_os = f'{system} {version}'
        request.headers.update({'HostOs': host_os})

    def _add_runtime_environment_header(self, request) -> None:
        """
        Add RuntimeEnvironment request header to capture the runtime framework
         on which the client SDK is running on.
        """
        python_version = platform.python_version()
        runtime_environment = f'Python/{python_version}'
        request.headers.update({'RuntimeEnvironment': runtime_environment})



from typing import TYPE_CHECKING, Optional, Union

from kiota_authentication_azure.azure_identity_authentication_provider import (
    AzureIdentityAuthenticationProvider as KiotaAzureIdentityAuthenticationProvider,
)

from .._enums import NationalClouds

if TYPE_CHECKING:
    from azure.core.credentials import TokenCredential
    from azure.core.credentials_async import AsyncTokenCredential


class AzureIdentityAuthenticationProvider(KiotaAzureIdentityAuthenticationProvider):

    def __init__(
        self,
        credentials: Union["TokenCredential", "AsyncTokenCredential"],
        options: Optional[dict] = {},
        scopes: list[str] = [],
        allowed_hosts: list[str] = [nc.value for nc in NationalClouds]
    ) -> None:
        """[summary]

        Args:
            credentials (Union["TokenCredential", "AsyncTokenCredential"]): The
                tokenCredential implementation to use for authentication.
            options (Optional[dict]): The options to use for authentication.
            scopes (list[str]): The scopes to use for authentication.
                Defaults to 'https://<national_cloud_domain>/.default'.
            allowed_hosts (Optional[list[str]]): The allowed hosts to use for
                authentication. Defaults to Microsoft National Clouds.
        """
        super().__init__(credentials, options, scopes, allowed_hosts)



import json

import httpx
from kiota_http.middleware import MiddlewarePipeline, RedirectHandler, RetryHandler

from .._enums import FeatureUsageFlag
from .request_context import MicrosoftAgentsM365CopilotRequestContext


class AsyncMicrosoftAgentsM365CopilotTransport(httpx.AsyncBaseTransport):
    """A custom transport for requests to the Microsoft Agents M365 Copilot API"""

    def __init__(self, transport: httpx.AsyncBaseTransport, pipeline: MiddlewarePipeline) -> None:
        self.transport = transport
        self.pipeline = pipeline

    async def handle_async_request(self, request: httpx.Request) -> httpx.Response:
        if self.pipeline and hasattr(request, 'options'):
            self.set_request_context_and_feature_usage(request)
            response = await self.pipeline.send(request)
            return response

        response = await self.transport.handle_async_request(request)
        return response

    def set_request_context_and_feature_usage(self, request: httpx.Request) -> httpx.Request:

        request_options = request.options  # type:ignore

        context = MicrosoftAgentsM365CopilotRequestContext(request_options, request.headers)
        middleware = self.pipeline._first_middleware
        while middleware:
            if isinstance(middleware, RedirectHandler):
                context.feature_usage = FeatureUsageFlag.REDIRECT_HANDLER_ENABLED
            if isinstance(middleware, RetryHandler):
                context.feature_usage = FeatureUsageFlag.RETRY_HANDLER_ENABLED

            middleware = middleware.next
        request.context = context  #type: ignore
        return request



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from __future__ import annotations

from typing import Optional

import httpx
from kiota_abstractions.request_option import RequestOption
from kiota_http.kiota_client_factory import KiotaClientFactory
from kiota_http.middleware.middleware import BaseMiddleware

from ._enums import APIVersion, NationalClouds
from .middleware import (
    AsyncMicrosoftAgentsM365CopilotTransport,
    MicrosoftAgentsM365CopilotTelemetryHandler,
    MicrosoftAgentsM365CopilotTelemetryHandlerOption,
)


class MicrosoftAgentsM365CopilotClientFactory(KiotaClientFactory):
    """Constructs httpx.AsyncClient instances configured with either custom or default
    pipeline of graph specific middleware.
    """

    @staticmethod
    def create_with_default_middleware( # type: ignore
        # Breaking change to remove KiotaClientFactory as base class
        api_version: APIVersion = APIVersion.v1,
        client: Optional[httpx.AsyncClient] = None,
        host: NationalClouds = NationalClouds.Global,
        options: Optional[dict[str, RequestOption]] = None
    ) -> httpx.AsyncClient:
        """Constructs native HTTP AsyncClient(httpx.AsyncClient) instances configured with
        a custom transport loaded with a default pipeline of middleware.

        Args:
            api_version (APIVersion): The Graph API version to be used.
            Defaults to APIVersion.v1.
            client  (httpx.AsyncClient): The httpx.AsyncClient instance to be used.
            Defaults to KiotaClientFactory.get_default_client().
            host (NationalClouds): The national clound endpoint to be used.
            Defaults to NationalClouds.Global.
            options (Optional[dict[str, RequestOption]]): The request options to use when
            instantiating default middleware. Defaults to dict[str, RequestOption]=None.

        Returns:
            httpx.AsyncClient: An instance of the AsyncClient object
        """
        if client is None:
            client = KiotaClientFactory.get_default_client()
        base_url = MicrosoftAgentsM365CopilotClientFactory._get_base_url(host, api_version)
        client.base_url = base_url
        middleware = KiotaClientFactory.get_default_middleware(options)
        telemetry_handler = MicrosoftAgentsM365CopilotClientFactory._get_telemetry_handler(options)
        middleware.append(telemetry_handler)
        return MicrosoftAgentsM365CopilotClientFactory._load_middleware_to_client(
            client, middleware
        )

    @staticmethod
    def create_with_custom_middleware( # type: ignore
        # Breaking change to remove Kiota client factory as base class
        middleware: Optional[list[BaseMiddleware]],
        api_version: APIVersion = APIVersion.v1,
        client: Optional[httpx.AsyncClient] = None,
        host: NationalClouds = NationalClouds.Global,
    ) -> httpx.AsyncClient:
        """Applies a custom middleware chain to the HTTP Client

        Args:
            middleware(list[BaseMiddleware]): Custom middleware list that will be used to create
            a middleware pipeline. The middleware should be arranged in the order in which they will
            modify the request.
            api_version (APIVersion): The Graph API version to be used.
            Defaults to APIVersion.v1.
            client  (httpx.AsyncClient): The httpx.AsyncClient instance to be used.
            Defaults to KiotaClientFactory.get_default_client().
            host (NationalClouds): The national clound endpoint to be used.
            Defaults to NationalClouds.Global.
        """
        if client is None:
            client = KiotaClientFactory.get_default_client()
        base_url = MicrosoftAgentsM365CopilotClientFactory._get_base_url(host, api_version)
        client.base_url = base_url
        return MicrosoftAgentsM365CopilotClientFactory._load_middleware_to_client(
            client, middleware
        )

    @staticmethod
    def _get_base_url(host: str, api_version: APIVersion) -> str:
        """Helper method to set the complete base url"""
        base_url = f'{host}/{api_version}'
        return base_url

    @staticmethod
    def _get_telemetry_handler(
        options: Optional[dict[str, RequestOption]]
    ) -> MicrosoftAgentsM365CopilotTelemetryHandler:
        """Helper method to get the telemetry handler instantiated with appropriate options"""

        if options:
            telemetry_options: MicrosoftAgentsM365CopilotTelemetryHandlerOption = options.get(
                MicrosoftAgentsM365CopilotTelemetryHandlerOption().get_key()
            )  # type: ignore # Unable to down cast type
            if telemetry_options:
                return MicrosoftAgentsM365CopilotTelemetryHandler(options=telemetry_options)
        return MicrosoftAgentsM365CopilotTelemetryHandler()

    @staticmethod
    def _load_middleware_to_client(
        client: httpx.AsyncClient, middleware: Optional[list[BaseMiddleware]]
    ) -> httpx.AsyncClient:
        current_transport = client._transport
        client._transport = MicrosoftAgentsM365CopilotClientFactory._replace_transport_with_custom_transport(
            current_transport, middleware
        )
        if client._mounts:
            mounts: dict = {}
            for pattern, transport in client._mounts.items():
                if transport is None:
                    mounts[pattern] = None
                else:
                    ps = MicrosoftAgentsM365CopilotClientFactory._replace_transport_with_custom_transport(
                        transport, middleware
                    )
                    mounts[pattern] = ps
            client._mounts = dict(sorted(mounts.items()))
        return client

    @staticmethod
    def _replace_transport_with_custom_transport(
        current_transport: httpx.AsyncBaseTransport, middleware: Optional[list[BaseMiddleware]]
    ) -> AsyncMicrosoftAgentsM365CopilotTransport:
        middleware_pipeline = KiotaClientFactory.create_middleware_pipeline(
            middleware, current_transport
        )
        new_transport = AsyncMicrosoftAgentsM365CopilotTransport(
            transport=current_transport, pipeline=middleware_pipeline
        )
        return new_transport



from typing import Optional

from kiota_abstractions.request_option import RequestOption

from ..._constants import SDK_VERSION
from ..._enums import APIVersion


class MicrosoftAgentsM365CopilotTelemetryHandlerOption(RequestOption):
    """Config options for the MicrosoftAgentsM365CopilotTelemetryHandlerOption"""

    AGENTS_COPILOT_TELEMETRY_HANDLER_OPTION_KEY = "MicrosoftAgentsM365CopilotTelemetryHandlerOption"

    def __init__(
        self, api_version: Optional[APIVersion] = None, sdk_version: str = SDK_VERSION
    ) -> None:
        """To create an instance of MicrosoftAgentsM365CopilotTelemetryHandlerOption

        Args:
            api_version (Optional[APIVersion], optional): The M365 copilot API version in use.
            Defaults to None.
            sdk_version (str): The sdk version in use.
            Defaults to SDK_VERSION of microsoft_agents_m365copilot_core.
        """
        self._api_version = api_version
        self._sdk_version = sdk_version

    @property
    def api_version(self):
        """The M365 copilot API version in use"""
        return self._api_version

    @api_version.setter
    def api_version(self, value: APIVersion):
        self._api_version = value

    @property
    def sdk_version(self):
        """The sdk version in use"""
        return self._sdk_version

    @sdk_version.setter
    def sdk_version(self, value: str):
        self._sdk_version = value

    @staticmethod
    def get_key() -> str:
        return MicrosoftAgentsM365CopilotTelemetryHandlerOption.AGENTS_COPILOT_TELEMETRY_HANDLER_OPTION_KEY



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from .async_transport import AsyncMicrosoftAgentsM365CopilotTransport
from .options import MicrosoftAgentsM365CopilotTelemetryHandlerOption
from .request_context import MicrosoftAgentsM365CopilotRequestContext
from .telemetry import MicrosoftAgentsM365CopilotTelemetryHandler

__all__ = [
    "MicrosoftAgentsM365CopilotTelemetryHandlerOption", "MicrosoftAgentsM365CopilotRequestContext",
    "AsyncMicrosoftAgentsM365CopilotTransport", "MicrosoftAgentsM365CopilotTelemetryHandler"
]



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
#pylint: disable=invalid-name

from enum import Enum


class APIVersion(str, Enum):
    """Enumerated list of supported API Versions"""
    beta = 'beta'
    v1 = 'v1.0'

    def __str__(self):
        return self.value


class FeatureUsageFlag(int, Enum):
    """Enumerated list of values used to flag usage of specific middleware"""

    NONE = 0
    REDIRECT_HANDLER_ENABLED = 1
    RETRY_HANDLER_ENABLED = 2
    AUTH_HANDLER_ENABLED = 4
    DEFAULT_HTTP_PROVIDER_ENABLED = 8
    LOGGING_HANDLER_ENABLED = 16

    def __str__(self):
        return self.value


class NationalClouds(str, Enum):
    """Enumerated list of supported sovereign clouds"""

    China = 'https://microsoftgraph.chinacloudapi.cn'
    Germany = 'https://graph.microsoft.de'
    Global = 'https://graph.microsoft.com'
    US_DoD = 'https://dod-graph.microsoft.us'
    US_GOV = 'https://graph.microsoft.us'

    def __str__(self):
        return self.value
